/*
This file is part of GNU Zebra.
Copyright (C) 1999 Robert Olsson, Jens Ls

This work is based work with the following copyright notice:

 OSPF network related functions
 Copyright (C) 1999 Toshiaki Takada

GNU Zebra is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

GNU Zebra is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU Zebra; see the file COPYING.  If not, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.  */

#include <zebra.h>
#include <memory.h>
#include <stream.h>
#include "thread.h"
#include "linklist.h"
#include "prefix.h"
#include "if.h"
#include "sockunion.h"
#include "log.h"
#include "pim.h"
#include "pimd.h"


int pim_sock, igmp_sock;

/* Make pimd's server socket. */
int
pim_serv_sock (int family)
{
  int ret, i;

  pim_sock = socket (family, SOCK_RAW, IPPROTO_PIM);
  if (pim_sock < 0)
    return pim_sock;
  
  i = 1;
  ret = setsockopt (pim_sock, IPPROTO_IP, IP_TTL, 
                        (void *) &i, sizeof (i));
  if (ret < 0)
    return ret;
  
  i = 1;
  ret = setsockopt (pim_sock, IPPROTO_IP, IP_PKTINFO, 
                        (void *) &i, sizeof (i));

  if (ret < 0)
    return ret;

  return pim_sock;
}
/* Make igmp's server socket. */
int
igmp_serv_sock (int family)
{
  int ret, i;

  igmp_sock = socket (family, SOCK_RAW, IPPROTO_IGMP);
  if (igmp_sock < 0)
    return igmp_sock;

  i = 1;
  ret = setsockopt (igmp_sock, IPPROTO_IP, IP_TTL, 
                        (void *) &i, sizeof (i));
  if (ret < 0)
    return ret;

  i = 1;
  ret = setsockopt (igmp_sock, IPPROTO_IP, IP_PKTINFO, 
                        (void *) &i, sizeof (i));

  if (ret < 0)
    return ret;

  return igmp_sock;
}

void
socket_init()
{
  /* Create raw socket. */
  pim_sock = pim_serv_sock (AF_INET);
  if (pim_sock < 0)
    {
      zlog_warn ("can't create pim socket %s", strerror(errno));
      return;
    }
  t_pim_raw = thread_add_read (master, pim_read_raw, NULL, pim_sock);

  /* Create raw socket. */
  igmp_sock = igmp_serv_sock (AF_INET);
  if (igmp_sock < 0)
    {
      zlog_warn ("can't create igmp raw socket %s", strerror(errno));
      return;
    }
  t_igmp_raw = thread_add_read (master, igmp_read_raw, NULL, igmp_sock);

}
void
data_timer_init()
{
  u_int32_t timer = DATA_TIMER_INTERVAL;
  t_pim_data = thread_add_timer (master, data_timer_thread, NULL, timer);
}

/* Join to the add/leave multicast group. */

int
if_group (struct interface *ifp, int sock, u_int32_t group, int add_leave)
{
  struct pim_interface *pi;
  struct ip_mreq m;
  int ret;

  pi = ifp->info;

  bzero (&m, sizeof (m));
  m.imr_multiaddr.s_addr = htonl (group);
  m.imr_interface = pi->address->u.prefix4;

  ret = setsockopt (sock, IPPROTO_IP, add_leave,
	      (char *) &m, sizeof (struct ip_mreq));

  if (ret < 0)
    zlog_warn ("can't setsockopt %u: %s",
	       group,
	       strerror (errno));

  return ret;
}

int
if_add_groups (struct interface *ifp)
{
  int ret;

  ret = if_group (ifp, pim_sock, INADDR_PIM_ALLRTRS_GROUP, IP_ADD_MEMBERSHIP);
  if (ret < 0)
    return ret;

  ret = if_group (ifp, igmp_sock, INADDR_ALLRTRS_GROUP, IP_ADD_MEMBERSHIP);
  if (ret < 0)
    return ret;
  
  return 0;
}
int

if_drop_groups (struct interface *ifp)
{
  int ret;

  ret = if_group (ifp, pim_sock, INADDR_PIM_ALLRTRS_GROUP, IP_DROP_MEMBERSHIP);
  if (ret < 0)
    return ret;

  ret = if_group (ifp, igmp_sock, INADDR_ALLRTRS_GROUP, IP_DROP_MEMBERSHIP);
  if (ret < 0)
    return ret;
  
  return 0;
}

